using System;
using System.Collections;
using System.Collections.Generic;
using System.Globalization;
using System.Text;

namespace Vevo.Shared.Utilities
{
    public static class StringUtilities
    {
        public static string GetNameByLength( string name, int length )
        {
            if (name.Length <= length)
                return name;
            else
                return name.Substring( 0, length ) + "..";
        }

        // Trim preceding and trailing spaces. The final text will be trimmed to the maxLength
        public static string Trim( string text, int maxLength )
        {
            string result = text.Trim();

            if (result.Length > maxLength)
                return result.Substring( 0, maxLength );
            else
                return result;
        }

        public static int CountOccurrencesInString( string source, char value )
        {
            int count = 0;
            for (int i = 0; i < source.Length; i++)
                if (source[i] == value)
                    count++;
            return count;
        }

        public static int CountOccurrencesInString( string source, string subString )
        {
            int count = 0;
            int curIndex = 0;
            do
            {
                curIndex = source.IndexOf( subString, curIndex );
                if (curIndex != -1)
                {
                    count++;
                    curIndex += subString.Length;
                }
            }
            while (curIndex != -1);

            return count;
        }

        public static string[] SplitString( string source, params char[] separators )
        {
            string[] list = source.Split( separators,
                StringSplitOptions.RemoveEmptyEntries );
            ArrayList array = new ArrayList();
            foreach (string item in list)
            {
                if (item != null)
                {
                    string trimmed = item.Trim();
                    if (trimmed.Length > 0)
                        array.Add( trimmed );
                }
            }

            string[] result = new string[array.Count];
            array.CopyTo( result );
            return result;
        }

        public static bool IsUnicodeLetterOrDigit( char c )
        {
            switch (CharUnicodeInfo.GetUnicodeCategory( c ))
            {
                case UnicodeCategory.UppercaseLetter:
                case UnicodeCategory.LowercaseLetter:
                case UnicodeCategory.TitlecaseLetter:
                case UnicodeCategory.LetterNumber:
                case UnicodeCategory.OtherLetter:
                case UnicodeCategory.DecimalDigitNumber:
                case UnicodeCategory.ModifierLetter:
                case UnicodeCategory.NonSpacingMark:
                    return true;

                default:
                    return false;
            }
        }

        public static String RemoveDiacritics( String s )
        {
            String normalizedString = s.Normalize( NormalizationForm.FormD );
            StringBuilder stringBuilder = new StringBuilder();

            for (int i = 0; i < normalizedString.Length; i++)
            {
                Char c = normalizedString[i];
                if (CharUnicodeInfo.GetUnicodeCategory( c ) != UnicodeCategory.NonSpacingMark)
                    stringBuilder.Append( c );
            }

            return stringBuilder.ToString();
        }

        public static bool IsStringInArray( string[] array, string text, bool ignoreCase )
        {
            if (array != null)
            {
                foreach (string content in array)
                {
                    if (String.Compare( content, text, ignoreCase ) == 0)
                        return true;
                }
            }

            return false;
        }

        public static bool ContainsNumber( string source )
        {
            foreach (Char ch in source)
            {
                if (Char.IsDigit( ch ))
                    return true;
            }
            return false;
        }

        public static bool ContainsAlphabet( string source )
        {
            foreach (Char ch in source)
            {
                if (Char.IsLetter( ch ))
                    return true;
            }
            return false;
        }

        public static bool ContainsSymbol( string source )
        {
            foreach (Char ch in source)
            {
                if (Char.IsPunctuation( ch ) || Char.IsSymbol( ch ))
                    return true;
            }
            return false;
        }

        public static bool ContainsLowercase( string source )
        {
            foreach (Char ch in source)
            {
                if (Char.IsLower( ch ))
                    return true;
            }
            return false;
        }

        public static bool ContainsUppercase( string source )
        {
            foreach (Char ch in source)
            {
                if (Char.IsUpper( ch ))
                    return true;
            }
            return false;
        }
    }
}
