using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.IO;
using System.Text;
using System.Web;
using System.Web.UI.WebControls;
using System.Data;
using Vevo.Shared.SystemServices;

namespace Vevo.Shared.WebUI
{
    /// <summary>
    /// Summary description for FolderPermissionTest
    /// </summary>
    public class ApplicationFolderPermissionTester
    {
        #region Private

        private const string WriteTestFileName = "VevoWriteTestFile.txt";

        private String[] _testFolders;

        private bool _writeSuccess = true;
        private bool _listSuccess = true;
        private StringCollection _writeErrorPaths = new StringCollection();
        private StringCollection _listErrorPaths = new StringCollection();

        private void TestFolder( string path )
        {
            string localPath = HttpContext.Current.Server.MapPath( path );
            if (localPath.ToLower().Contains( "install\\" ))
            {
                localPath = localPath.ToLower().Replace( "install\\", "" );
            }
            string fullpath = localPath + "\\" + WriteTestFileName;
            try
            {
                if (Directory.Exists( localPath ))
                {
                    if (File.Exists( fullpath ))
                    {
                        File.Delete( fullpath );
                    }

                    FileStream file = new FileStream( fullpath, FileMode.Create );
                    file.Close();
                    File.Delete( fullpath );

                    _writeSuccess &= true;
                }
                else
                {
                    _writeErrorPaths.Add( localPath );

                    _writeSuccess &= false;
                }
            }
            catch
            {
                _writeErrorPaths.Add( localPath );

                _writeSuccess &= false;
            }
        }

        private bool TestWrite( string folder )
        {
            _writeSuccess = true;
            TestFolder( folder );
            return _writeSuccess;
        }

        private bool TestListOneFolder( string path )
        {
            try
            {
                FileManager fileManager = new FileManager();
                string[] files = fileManager.SearchFiles( "~/" + path );

                if (files.Length > 0)
                {
                    return true;
                }
                else
                {
                    ProcessListFailure( path );
                    return false;
                }
            }
            catch
            {
                ProcessListFailure( path );
                return false;
            }
        }

        private void ProcessListFailure( string path )
        {
            _listErrorPaths.Add( HttpContext.Current.Server.MapPath( path ) );
            _listSuccess = false;
        }

        #endregion

        #region Constructor
        public ApplicationFolderPermissionTester( String[] FolderName )
        {
            _testFolders = new String[FolderName.Length];
            FolderName.CopyTo( _testFolders, 0 );
        }

        #endregion

        #region Public Properties

        public string WriteErrorMessage
        {
            get
            {
                string message = String.Empty;
                foreach (string path in _writeErrorPaths)
                {
                    message += path + "<br />";
                }

                return message;
            }
        }

        public string ListErrorMessage
        {
            get
            {
                StringBuilder sb = new StringBuilder();
                foreach (string path in _listErrorPaths)
                {
                    sb.Append( path );
                    sb.Append( "<br />" );
                }

                return sb.ToString();
            }
        }

        #endregion


        #region Public Methods

        public DataTable CreateTestFolderTable( bool isVerify )
        {
            bool permissionPass;
            DataTable table = CreateTestFolderTable( isVerify, out permissionPass );
            return table;
        }

        public DataTable CreateTestFolderTable( bool isVerify, out bool permissionPass )
        {
            DataTable dt = new DataTable();
            DataRow dr;

            dt.Columns.Add( new DataColumn( "Folder", typeof( string ) ) );
            dt.Columns.Add( new DataColumn( "Permission", typeof( string ) ) );

            permissionPass = true;

            for (int i = 0; i < _testFolders.Length; i++)
            {
                dr = dt.NewRow();
                dr[0] = _testFolders[i].ToString();
                if (!isVerify)
                {
                    dr[1] = "";
                }
                else
                {
                    if (TestWrite( _testFolders[i] )
                        && TestListOneFolder( _testFolders[i] ))
                    {
                        dr[1] = "pass";
                    }
                    else
                    {
                        dr[1] = "fail";
                        permissionPass = false;
                    }
                }
                dt.Rows.Add( dr );
            }

            return dt;
        }

        public bool TestWrite()
        {
            foreach (string path in _testFolders)
            {
                TestFolder( path );
            }
            return _writeSuccess;
        }

        public bool TestListFolderContents()
        {
            foreach (string path in _testFolders)
            {
                _listSuccess &= TestListOneFolder( path );
            }
            return _listSuccess;
        }

        #endregion

    }
}
