using System;
using System.Data;
using System.Text.RegularExpressions;
using System.Web;
using System.Web.UI;
using Vevo.Domain;
using Vevo.Shared.WebUI;
using Vevo.UrlModuleLib;
using Vevo.UrlModuleLib.Configuration;


namespace Vevo
{
    /// <summary>
    /// Summary description for UrlManager
    /// </summary>
    public static class UrlManager
    {
        private static string _incomingRulesPath = "~/DataFiles/UrlRewrite/IncomingRules.xml";
        private static string _outgoingRulesPath = "~/DataFiles/UrlRewrite/OutgoingRules.xml";

        private static string _mobileAgentRegexFilePath = "~/DataFiles/MobileAgentRegex.xml";

        private static UrlProcessor _urlProcessor;

        private static Regex _mobileBrowserRegex;
        private static Regex _mobileAppsRegex;

        private static void ReadMobileRegexFromFiles()
        {
            if ((_mobileBrowserRegex != null) && (_mobileAppsRegex != null))
                return;

            DataSet ds = new DataSet();
            ds.ReadXml( HttpContext.Current.Server.MapPath( _mobileAgentRegexFilePath ) );
            DataTable table = ds.Tables[0];
            for (int i = 0; i <= table.Rows.Count - 1; i++)
            {
                if (table.Rows[i]["Name"].ToString().Trim() == "MobileBrowser")
                {
                    _mobileBrowserRegex = new Regex( @table.Rows[i]["Rule"].ToString().Trim(), RegexOptions.IgnoreCase | RegexOptions.Multiline );
                }
                else if (table.Rows[i]["Name"].ToString().Trim() == "MobileApps")
                {
                    _mobileAppsRegex = new Regex( @table.Rows[i]["Rule"].ToString().Trim(), RegexOptions.IgnoreCase | RegexOptions.Multiline );
                }
            }
        }

        public static bool IsMobileClient()
        {
            HttpRequest request = HttpContext.Current.Request;
            HttpResponse response = HttpContext.Current.Response;

            bool isMobile = false;
            bool isCookieSet = false;
            var viewMobileCookie = request.Cookies[SystemConst.MobileViewBrowserCookieName];

            if (viewMobileCookie != null && bool.TryParse( viewMobileCookie.Value, out isMobile ))
            {
                isCookieSet = true;
            }
            else if (request.Browser.IsMobileDevice)
            {
                isMobile = true;
            }
            else if (request.ServerVariables["HTTP_X_WAP_PROFILE"] != null)
            {
                isMobile = true;
            }
            else if (request.ServerVariables["HTTP_ACCEPT"] != null && (request.ServerVariables["HTTP_ACCEPT"].ToLower().Contains( "wap" ) || request.ServerVariables["HTTP_ACCEPT"].ToLower().Contains( "wml+xml" )))
            {
                isMobile = true;
            }
            else if (request.ServerVariables["HTTP_USER_AGENT"] != null)
            {
                string userAgent = request.ServerVariables["HTTP_USER_AGENT"];
                ReadMobileRegexFromFiles();
                isMobile = ((_mobileBrowserRegex.IsMatch( userAgent ) || _mobileAppsRegex.IsMatch( userAgent.Substring( 0, 4 ) )));
            }

            if (!isCookieSet)
                response.Cookies.Add( new HttpCookie( SystemConst.MobileViewBrowserCookieName, isMobile.ToString() ) );

            return isMobile;
        }

        public static bool IsMobileDevice( HttpRequest Request )
        {
            string strUserAgent = Request.UserAgent.ToString().ToLower();
            bool isMobileDevice = Request.Browser.IsMobileDevice;
            if (Request.Cookies["MobileDevice"] != null)
            {
                if (Request.Cookies["MobileDevice"].Value == "IgnoreMobile") { return false; }
                else { return false; }
            }
            else
            {
                if (strUserAgent != null)
                {
                    if (isMobileDevice == true || strUserAgent.Contains( "iphone" ) || strUserAgent.Contains( "ipad" ) || strUserAgent.Contains( "blackberry" ) || strUserAgent.Contains( "mobile" ) ||
                    strUserAgent.Contains( "android" ) || strUserAgent.Contains( "windows ce" ) || strUserAgent.Contains( "opera mini" ) || strUserAgent.Contains( "palm" ))
                    {
                        return true;
                    }
                    else { return false; }

                }
                else { return false; }

            }
        }

        public static bool IsMobile()
        {
            if (HttpContext.Current == null)
                return false;
            foreach (string segments in HttpContext.Current.Request.Url.Segments)
            {
                if (segments.Substring( 0, segments.Length - 1 ).ToLower().Equals( MobileFolder.ToLower() ))
                    return true;
            }

            return false;
        }

        public static bool IsFacebook()
        {
            if (HttpContext.Current == null)
                return false;
            foreach (string segments in HttpContext.Current.Request.Url.Segments)
            {
                if (segments.Substring( 0, segments.Length - 1 ).ToLower().Equals( FacebookFolder.ToLower() ))
                    return true;
            }

            return false;
        }

        public static bool IsBlog()
        {
            if (HttpContext.Current == null)
                return false;
            foreach (string segments in HttpContext.Current.Request.Url.Segments)
            {
                if (segments.Substring( 0, segments.Length - 1 ).ToLower().Equals( BlogFolder.ToLower() ))
                    return true;
            }
            return false;
        }

        public static UrlProcessor Processor
        {
            get
            {
                string appPath = HttpContext.Current.Request.ApplicationPath;

                if (_urlProcessor == null)
                {
                    UrlConfiguration config = new UrlConfiguration( _incomingRulesPath, _outgoingRulesPath );
                    _urlProcessor = UrlProcessor.Create(
                        appPath,
                        appPath + "/" + MobileFolder,
                        config.GetConfig() );
                }

                return _urlProcessor;
            }
        }

        public static string MobileFolder
        {
            get
            {
                return "Mobile";
            }
        }

        public static string FacebookFolder
        {
            get
            {
                return "Facebook";
            }
        }
        public static string BlogFolder
        {
            get
            {
                return "Blog";
            }
        }

        public static string GetCategoryUrlSiteMap( object categoryID, object urlName )
        {
            if (DataAccessContext.Configurations.GetBoolValue( "UseSimpleCatalogUrl" ))
                return String.Format( "~/Catalog.aspx?CategoryName={0}", urlName );
            else
                return String.Format( "~/Catalog.aspx?CategoryID={0}", categoryID );
        }

        public static string GetDepartmentUrlSiteMap( object departmentID, object urlName )
        {
            if (DataAccessContext.Configurations.GetBoolValue( "UseSimpleCatalogUrl" ))
                return String.Format( "~/Department.aspx?DepartmentName={0}", urlName );
            else
                return String.Format( "~/Department.aspx?DepartmentID={0}", departmentID );
        }

        public static string GetProductUrlSiteMap( object productID, object urlName )
        {
            if (DataAccessContext.Configurations.GetBoolValue( "UseSimpleCatalogUrl" ))
                return String.Format( "~/Product.aspx?ProductName={0}", urlName );
            else
                return String.Format( "~/Product.aspx?ProductID={0}", productID );
        }

        public static string GetCategoryUrl( object categoryID, object urlName )
        {
            if (DataAccessContext.Configurations.GetBoolValue( "UseSimpleCatalogUrl" ))
                return UrlManager.Processor.ResolveOutgoingCategoryUrl( categoryID.ToString(), urlName.ToString() );
            else
                return "~/Catalog.aspx?CategoryID=" + categoryID.ToString();
        }

        public static string GetMobileCategoryUrl( object categoryID, object urlName )
        {
            if (DataAccessContext.Configurations.GetBoolValue( "UseSimpleCatalogUrl" ))
                return UrlManager.Processor.ResolveOutgoingCategoryUrl(
                    categoryID.ToString(), urlName.ToString() ).Replace( "~/", "~/" + MobileFolder + "/" );
            else
                return "~/" + MobileFolder + "/Catalog.aspx?CategoryID=" + categoryID.ToString();
        }

        public static string GetFacebookCategoryUrl( object categoryID, object urlName )
        {
            return "~/" + FacebookFolder + "/Default.aspx?Index=3&CategoryID=" + categoryID.ToString();
        }

        public static string GetDepartmentUrl( object departmentID, object urlName )
        {
            if (DataAccessContext.Configurations.GetBoolValue( "UseSimpleCatalogUrl" ))
                return UrlManager.Processor.ResolveOutgoingDepartmentUrl( departmentID.ToString(), urlName.ToString() );
            else
                return "~/Department.aspx?DepartmentID=" + departmentID.ToString();
        }

        public static string GetManufacturerUrl( object manufacturerID, object urlName )
        {
            if (DataAccessContext.Configurations.GetBoolValue( "UseSimpleCatalogUrl" ))
                return UrlManager.Processor.ResolveOutgoingManufacturerUrl( manufacturerID.ToString(), urlName.ToString() );
            else
                return "~/Manufacturer.aspx?ManufacturerID=" + manufacturerID.ToString();
        }
        public static string GetProductUrl( object productID, object urlName )
        {
            if (DataAccessContext.Configurations.GetBoolValue( "UseSimpleCatalogUrl" ))
                return UrlManager.Processor.ResolveOutgoingProductUrl( productID.ToString(), urlName.ToString() );
            else
                return "~/Product.aspx?ProductID=" + productID.ToString();
        }

        public static string GetMobileProductUrl( object productID, object urlName )
        {
            if (DataAccessContext.Configurations.GetBoolValue( "UseSimpleCatalogUrl" ))
                return UrlManager.Processor.ResolveOutgoingProductUrl(
                    productID.ToString(), urlName.ToString() ).Replace( "~/", "~/" + MobileFolder + "/" );
            else
                return "~/" + MobileFolder + "/Product.aspx?ProductID=" + productID.ToString();
        }

        public static string GetFacebookProductUrl( object productID, object urlName )
        {
            return "~/" + FacebookFolder + "/Default.aspx?Index=3&ProductID=" + productID.ToString();
        }

        public static string GetProductFullUrl( object productID, object urlName )
        {
            string url = GetProductUrl( productID, urlName );
            if (url.StartsWith( "~" ))
            {
                url = url.Substring( 2 );
            }
            return UrlPath.StorefrontUrl + url;
        }

        public static string GetTellFriendUrl( object productID, object urlName )
        {
            if (DataAccessContext.Configurations.GetBoolValue( "UseSimpleCatalogUrl" ))
                return UrlManager.Processor.ResolveOutgoingTellFriendUrl( urlName.ToString() );
            else
                return "~/TellFriend.aspx?ProductID=" + productID.ToString();
        }

        public static string GetPromotionTellFriendUrl( object promotionID, object urlName )
        {
            if (DataAccessContext.Configurations.GetBoolValue( "UseSimpleCatalogUrl" ))
                return UrlManager.Processor.ResolveOutgoingTellFriendUrl( urlName.ToString() );
            else
                return "~/TellFriend.aspx?PromotionID=" + promotionID.ToString();
        }

        public static string GetTellFriendMobileUrl( object productID, object urlName )
        {
            if (DataAccessContext.Configurations.GetBoolValue( "UseSimpleCatalogUrl" ))
                return UrlManager.Processor.ResolveOutgoingTellFriendUrl( urlName.ToString() ).Replace( "~/", "~/" + MobileFolder + "/" );
            else
                return "~/Mobile/TellFriend.aspx?ProductID=" + productID.ToString();
        }

        public static string GetPromotionTellFriendMobileUrl( object promotionID, object urlName )
        {
            if (DataAccessContext.Configurations.GetBoolValue( "UseSimpleCatalogUrl" ))
                return UrlManager.Processor.ResolveOutgoingTellFriendUrl( urlName.ToString() ).Replace( "~/", "~/" + MobileFolder + "/" );
            else
                return "~/Mobile/TellFriend.aspx?PromotionID=" + promotionID.ToString();
        }

        public static string GetCustomerReviewUrl( object productID, object urlName )
        {
            if (DataAccessContext.Configurations.GetBoolValue( "UseSimpleCatalogUrl" ))
                return UrlManager.Processor.ResolveOutgoingCustomerReviewUrl( urlName.ToString() );
            else
                return "~/CustomerReview.aspx?ProductID=" + productID.ToString();
        }

        public static string GetContentUrl( object contentID, object urlName )
        {
            if (DataAccessContext.Configurations.GetBoolValue( "UseSimpleCatalogUrl" ))
                return UrlManager.Processor.ResolveOutgoingContentUrl( contentID.ToString(), urlName.ToString() );
            else
                return "~/Content.aspx?ContentID=" + contentID.ToString();
        }

        public static string GetContentMenuUrl( object contentMenuItemID, object urlName )
        {
            if (DataAccessContext.Configurations.GetBoolValue( "UseSimpleCatalogUrl" ))
                return UrlManager.Processor.ResolveOutgoingContentMenuUrl( contentMenuItemID.ToString(), urlName.ToString() );
            else
                return "~/ContentMenuItem.aspx?ContentMenuItemID=" + contentMenuItemID.ToString();
        }
        public static string GetNewsUrl( object newsID, object urlName )
        {
            if (DataAccessContext.Configurations.GetBoolValue( "UseSimpleCatalogUrl" ))
                return UrlManager.Processor.ResolveOutgoingNewsUrl( newsID.ToString(), urlName.ToString() );
            else
                return "~/NewsDetails.aspx?NewsID=" + newsID.ToString();
        }
        public static string GetBlogUrl( object blogID, object urlName )
        {
            if (DataAccessContext.Configurations.GetBoolValue( "UseSimpleCatalogUrl" ))
                return UrlManager.Processor.ResolveOutgoingBlogUrl( blogID.ToString(), urlName.ToString() );
            else
                return "~/" + BlogFolder + "/BlogDetails.aspx?BlogID=" + blogID.ToString();
        }
        public static string GetBlogListUrl( object urlName )
        {
            if (DataAccessContext.Configurations.GetBoolValue( "UseSimpleCatalogUrl" ))
                return UrlManager.Processor.ResolveOutgoingBlogListUrl( urlName.ToString() );
            else
                return "~/" + BlogFolder + "/BlogList.aspx?BlogListName=" + urlName.ToString();
        }
        public static string GetBlogCategoryUrl( object urlName )
        {
            if (DataAccessContext.Configurations.GetBoolValue( "UseSimpleCatalogUrl" ))
                return UrlManager.Processor.ResolveOutgoingBlogCategoryUrl( urlName.ToString() );
            else
                return "~/" + BlogFolder + "/CategoryList.aspx?CategoryName=" + urlName.ToString();
        }
        public static string GetPromotionUrl( object promotionID, object urlName )
        {
            if (DataAccessContext.Configurations.GetBoolValue( "UseSimpleCatalogUrl" ))
                return UrlManager.Processor.ResolveOutgoingPromotionUrl( promotionID.ToString(), urlName.ToString() );
            else
                return "~/PromotionDetail.aspx?PromoID=" + promotionID.ToString();
        }

        public static string GetMobilePromotionUrl( object promotionID, object urlName )
        {
            if (DataAccessContext.Configurations.GetBoolValue( "UseSimpleCatalogUrl" ))
                return UrlManager.Processor.ResolveOutgoingPromotionUrl(
                    promotionID.ToString(), urlName.ToString() ).Replace( "~/", "~/" + MobileFolder + "/" );
            else
                return "~/" + MobileFolder + "/PromotionDetail.aspx?PromoID=" + promotionID.ToString();
        }

        public static string GetFacebookPromotionUrl( object promotionID, object urlName )
        {
            return "~/" + FacebookFolder + "/Default.aspx?Index=2&PromoID=" + promotionID.ToString();
        }
    }
}
