using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;

namespace Vevo.Domain.Payments.SessionState
{
    public class SessionPool
    {
        private const int VerifyExpiredSessionIntervalSeconds = 300;

        private static Dictionary<string, Session> _sessions;

        private static int _idleTimeoutSeconds;

        private static DateTime _lastVerifyExpiredSessionTime;


        private static void ResetVerifyExpiredSessionTime()
        {
            _lastVerifyExpiredSessionTime = DateTime.MinValue;
        }

        private static void UpdateVerifyExpiredSessionTime()
        {
            _lastVerifyExpiredSessionTime = DateTime.Now;
        }

        private static bool IsTimeToClear()
        {
            TimeSpan timePassed = DateTime.Now.Subtract( _lastVerifyExpiredSessionTime );

            return timePassed >= new TimeSpan( 0, 0, VerifyExpiredSessionIntervalSeconds );
        }

        private static void ClearExpiredSession()
        {
            if (!IsTimeToClear())
                return;

            UpdateVerifyExpiredSessionTime();

            List<string> expiredSessions = new List<string>();

            IDictionaryEnumerator enumerator = _sessions.GetEnumerator();
            while (enumerator.MoveNext())
            {
                Session session = (Session) enumerator.Value;
                if (session.IsExpired( IdleTimeoutSeconds ))
                    expiredSessions.Add( session.Ticket );
            }

            foreach (string ticket in expiredSessions)
            {
                _sessions.Remove( ticket );
            }
        }


        static SessionPool()
        {
            _sessions = new Dictionary<string, Session>();
            _idleTimeoutSeconds = DefaultIdleTimeoutSeconds;

            ResetVerifyExpiredSessionTime();
        }

        #region Public
        public const int DefaultIdleTimeoutSeconds = 30 * 60;      // 30 minutes

        public static int IdleTimeoutSeconds
        {
            get
            {
                return _idleTimeoutSeconds;
            }
            set
            {
                ResetVerifyExpiredSessionTime();

                _idleTimeoutSeconds = value;
            }
        }

        public static string CreateSession()
        {
            Session session = new Session();
            _sessions.Add( session.Ticket, session );

            return session.Ticket;
        }

        public static Session GetSession( string ticket )
        {
            ClearExpiredSession();

            string key = ticket.ToLower();

            if (!_sessions.ContainsKey( key ))
                return null;

            Session session = _sessions[key];
            session.RenewTimeout();

            return session;
        }
        #endregion
    }
}
