using System;
using System.Data;
using System.Collections;
using System.Collections.Generic;
using System.Configuration;
using System.Text;
using System.Globalization;

using Vevo;
using Vevo.Domain.Base;
using Vevo.Domain.Marketing;
using Vevo.Domain.Products;
using Vevo.Domain.Users;
using Vevo.Domain.Contents;
using Vevo.Base.Domain;

namespace Vevo.Domain.Orders
{
    /// <summary>
    /// Summary description for Cart
    /// </summary>
    public class Cart
    {
        #region Public Enum

        public enum UpdateResult
        {
            None,
            Created,
            Updated,
            Deleted
        }

        #endregion


        #region Private

        private string _cartID = Guid.NewGuid().ToString();
        private IEntityList<ICartItem> _cartItems;
        private DateTime _createDate = DateTime.Now;
        private DateTime _updateDate = DateTime.Now;


        private bool CompareCartItem(
            ICartItem item,
            string productID,
            OptionItemValueCollection options,
            CartItemGiftDetails giftDetails,
            decimal productCustomPrice,
            decimal EnteredPrice )
        {
            if (item.ProductID == productID
                && item.Options.Equals( options )
                && item.GiftDetails.Equals( giftDetails )
                && item.ProductCustomPrice.Equals( productCustomPrice )
                && item.EnteredPrice.Equals( EnteredPrice )
                )
            {
                return true;
            }
            else
            {
                return false;
            }

        }

        //private bool CompareCartItem(
        //    ICartItem item,
        //    IPromotionSelected promotionSelected )
        //{
        //    return item.PromotionSelected.Equals( promotionSelected );
        //}

        private bool CompareCartItem(
            ICartItem item,
            string productID,
            OptionItemValueCollection options,
            ProductKitItemValueCollection productKits,
            decimal productCustomPrice )
        {
            if (item.ProductID == productID
                && item.ProductKits.Equals( productKits )
                && item.Options.Equals( options )
                && item.ProductCustomPrice.Equals( productCustomPrice )
                )
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        private bool CompareCartItem(
            ICartItem item,
            string productID,
            OptionItemValueCollection options,
            ProductKitItemValueCollection productKits )
        {
            if (item.ProductID == productID
                && item.Options.Equals( options )
                && item.ProductKits.Equals( productKits )
                )
            {
                return true;
            }
            else
            {
                return false;
            }
        }


        private UpdateResult DeleteItem( ICartItem item )
        {
            if (item != null)
            {
                CartItems.Remove( item );
                return UpdateResult.Deleted;
            }
            else
            {
                return UpdateResult.None;
            }
        }

        private UpdateResult UpdateQuantity(
            ICartItem item,
            Product product,
            int quantity,
            OptionItemValueCollection optionItemCollection,
            CartItemGiftDetails giftDetails,
            string giftRegistryItemID )
        {
            if (quantity == 0 ||
                item != null)
            {
                return UpdateQuantity( item, quantity );
            }
            else
            {
                ICartItem newItem = new CartItem(
                    CartID,
                    product,
                    quantity,
                    //unitPrice,
                    optionItemCollection,
                    giftDetails
                    );

                CartItems.Add( newItem );

                return UpdateResult.Created;
            }
        }
        private UpdateResult UpdateQuantity(
            ICartItem item,
            Product product,
            int quantity,
            OptionItemValueCollection optionItemCollection,
            CartItemGiftDetails giftDetails )
        {
            return UpdateQuantity( item, product, quantity, optionItemCollection, giftDetails, 0, false );
        }

        private UpdateResult UpdateQuantity(
            ICartItem item,
            Product product,
            int quantity,
            OptionItemValueCollection optionItemCollection,
            CartItemGiftDetails giftDetails,
            decimal productCustomPrice,
            bool isSubscriptionable )
        {
            return UpdateQuantity( item, product, quantity, optionItemCollection, giftDetails, productCustomPrice, 0, isSubscriptionable );
        }

        private UpdateResult UpdateQuantity(
            ICartItem item,
            Product product,
            int quantity,
            OptionItemValueCollection optionItemCollection,
            CartItemGiftDetails giftDetails,
            decimal productCustomPrice,
            decimal EnteredPrice,
            bool isSubscriptionable )
        {
            if (quantity == 0 ||
                item != null)
            {
                return UpdateQuantity( item, quantity );
            }
            else
            {
                CartItem newItem;
                if (product.IsCustomPrice)
                {
                    newItem = new CartItem(
                        CartID,
                        product,
                        quantity,
                        optionItemCollection,
                        productCustomPrice
                        );
                }
                else
                {

                    newItem = new CartItem(
                        CartID,
                        product,
                        quantity,
                        optionItemCollection,
                        giftDetails
                        );

                    if (EnteredPrice != 0)
                    {
                        newItem.EnteredPrice = EnteredPrice;
                        newItem.IsEnterPrice = true;
                    }

                }
                newItem.IsSubscriptionable = isSubscriptionable;
                CartItems.Add( newItem );

                return UpdateResult.Created;
            }
        }

        private int FindSubProductAmountInProductKit( string productID )
        {
            int amount = 0;
            foreach (ICartItem cartItem in CartItems)
            {
                if (cartItem.IsProductKit)
                {
                    foreach (ProductKitItemValue value in cartItem.ProductKits)
                    {
                        if (productID == value.ProductID)
                            amount += value.Quantity * cartItem.Quantity;
                    }
                }
            }
            return amount;
        }

        //private int FindSubProductAmountInPromotion( string productID, string optionCombinationID )
        //{
        //    int amount = 0;
        //    foreach (ICartItem cartItem in CartItems)
        //    {
        //        if (cartItem.IsPromotion)
        //        {
        //            foreach (PromotionSelectedItem value in cartItem.PromotionSelected.PromotionSelectedItems)
        //            {
        //                if (productID == value.Product.ProductID)
        //                {
        //                    List<string> optionIDsList = new List<string>( value.ProductOptionIDs );
        //                    string optionItemIDs = string.Join( ",", optionIDsList.ToArray() );
        //                    if ((optionItemIDs == optionCombinationID) || ((optionItemIDs == String.Empty) && (optionCombinationID == "0")))
        //                        amount += value.GetPromotionProduct.Quantity * cartItem.Quantity;
        //                }
        //            }
        //        }
        //    }
        //    return amount;
        //}

        private string RecurringUnitConvert( string recurringUnit )
        {
            if (recurringUnit == "Day")
                return "day(s)";
            else if (recurringUnit == "Month")
                return "month(s)";
            else if (recurringUnit == "Year")
                return "year(s)";
            else
                return "";
        }

        #endregion

        #region protected



        protected DateTime CalcuateEndDate(
            int recurringInterval,
            string recurringIntervalUnit,
            int recurringIntervalNumberOfCycle,
            int recurringIntervalNumberOfTrialCycle )
        {
            DateTime dayResult = DateTime.Today;

            int allCycle = recurringIntervalNumberOfCycle + recurringIntervalNumberOfTrialCycle;

            switch (recurringIntervalUnit.ToLower())
            {
                case "day":
                    return dayResult.AddDays( recurringInterval * (allCycle - 1) );
                case "week":
                    return dayResult.AddDays( (recurringInterval * 7) * (allCycle - 1) );
                case "month":
                    return dayResult.AddMonths( recurringInterval * (allCycle - 1) );
                case "year":
                    return dayResult.AddYears( recurringInterval * (allCycle - 1) );
                default:
                    throw new Exception( "Unknow period unit: " + recurringIntervalUnit );
            }
        }



        #endregion


        #region Public Properties

        public static Cart Null
        {
            get { return NullCart.Instance; }
        }

        public virtual bool IsNull
        {
            get { return false; }
        }

        public string CartID { get { return _cartID.ToString(); } set { _cartID = value; } }
        public DateTime CreateDate { get { return _createDate; } set { _createDate = value; } }
        public DateTime UpdateDate { get { return _updateDate; } set { _updateDate = value; } }

        public IEntityList<ICartItem> CartItems
        {
            get
            {
                if (_cartItems == null)
                    _cartItems = new EntityList<ICartItem>(
                        DataAccessContext.CartItemRepository.GetAllByCartID( CartID ) );

                return _cartItems;
            }
        }

        #endregion


        #region Public Methods

        public ICartItem FindItemInCart(
            string productID,
            OptionItemValueCollection options,
            CartItemGiftDetails giftDetails,
            decimal productCustomPrice )
        {
            return FindItemInCart( productID, options, giftDetails, productCustomPrice, 0.0m );
        }

        public ICartItem FindItemInCart(
            string productID,
            OptionItemValueCollection options,
            CartItemGiftDetails giftDetails,
            decimal productCustomPrice,
            decimal EnteredPrice )
        {
            foreach (ICartItem item in CartItems)
            {
                if (CompareCartItem( item, productID, options, giftDetails, productCustomPrice, EnteredPrice ))
                {
                    return item;
                }
            }
            return null;
        }

        public ICartItem FindItemInCart(
            string productID,
            OptionItemValueCollection options,
            ProductKitItemValueCollection productKits )
        {
            foreach (ICartItem item in CartItems)
            {
                if (CompareCartItem( item, productID, options, productKits ))
                {
                    return item;
                }
            }
            return null;
        }

        public ICartItem FindItemInCart(
            string productID,
            OptionItemValueCollection options,
            ProductKitItemValueCollection productKits,
            decimal productCustomPrice )
        {
            foreach (ICartItem item in CartItems)
            {
                if (CompareCartItem( item, productID, options, productKits, productCustomPrice ))
                {
                    return item;
                }
            }
            return null;
        }

        public ICartItem FindItemInCart( string cartItemID )
        {
            foreach (ICartItem item in CartItems)
            {
                if (item.CartItemID == cartItemID)
                    return item;
            }

            return null;
        }

        public int GetCurrentQuantity(
            string productID,
            OptionItemValueCollection optionItemIDs,
            CartItemGiftDetails giftDetails,
            decimal productCustomPrice )
        {
            ICartItem item = FindItemInCart(
                productID, optionItemIDs, giftDetails, productCustomPrice );

            if (item != null)
                return item.Quantity;
            else
                return 0;
        }

        public int GetCurrentQuantity(
           string productID,
           OptionItemValueCollection optionItemIDs,
           CartItemGiftDetails giftDetails,
           decimal productCustomPrice,
           decimal EnteredPrice )
        {
            ICartItem item = FindItemInCart(
                productID, optionItemIDs, giftDetails, productCustomPrice, EnteredPrice );

            if (item != null)
                return item.Quantity;
            else
                return 0;
        }

        public int GetCurrentQuantity(
            string productID,
            OptionItemValueCollection optionItemIDs,
            ProductKitItemValueCollection valueCollection )
        {
            ICartItem item = FindItemInCart(
                productID, optionItemIDs, valueCollection );

            if (item != null)
                return item.Quantity;
            else
                return 0;
        }

        public int GetCurrentQuantity(
            string productID,
            OptionItemValueCollection optionItemIDs,
            ProductKitItemValueCollection valueCollection,
            decimal productCustomPrice )
        {
            ICartItem item = FindItemInCart(
                productID, optionItemIDs, valueCollection, productCustomPrice );

            if (item != null)
                return item.Quantity;
            else
                return 0;
        }

        public UpdateResult UpdateQuantity(
            Product product,
            int quantity,
            OptionItemValueCollection optionItemCollection,
            CartItemGiftDetails giftDetails,
            decimal productCustomPrice,
            bool isSubscriptionable )
        {
            ICartItem item = FindItemInCart(
                product.ProductID, optionItemCollection, giftDetails, productCustomPrice );

            return UpdateQuantity(
                item, product, quantity, optionItemCollection, giftDetails, productCustomPrice, isSubscriptionable );
        }


        public UpdateResult UpdateQuantity(
            Product product,
            int quantity,
            OptionItemValueCollection optionItemCollection,
            CartItemGiftDetails giftDetails,
            decimal productCustomPrice,
            decimal EnteredPrice,
            bool isSubscriptionable )
        {
            ICartItem item = FindItemInCart(
                product.ProductID, optionItemCollection, giftDetails, productCustomPrice, EnteredPrice );

            return UpdateQuantity(
                item, product, quantity, optionItemCollection, giftDetails, productCustomPrice, EnteredPrice, isSubscriptionable );
        }

        public UpdateResult UpdateQuantity(
           Product product,
           int quantity,
           OptionItemValueCollection optionItemCollection,
           ProductKitItemValueCollection valueCollection,
           decimal productCustomPrice )
        {
            ICartItem item = FindItemInCart(
                product.ProductID, optionItemCollection, valueCollection, productCustomPrice );

            if (quantity == 0 ||
              item != null)
            {
                return UpdateQuantity( item, quantity );
            }
            else
            {
                CartItemProductKit newItem = new CartItemProductKit( CartID, product, optionItemCollection, valueCollection, quantity, productCustomPrice );
                CartItems.Add( newItem );

                return UpdateResult.Created;
            }
        }

        public UpdateResult UpdateQuantity(
            Product product,
            int quantity,
            OptionItemValueCollection optionItemCollection,
            ProductKitItemValueCollection valueCollection )
        {
            ICartItem item = FindItemInCart(
                product.ProductID, optionItemCollection, valueCollection );

            if (quantity == 0 ||
              item != null)
            {
                return UpdateQuantity( item, quantity );
            }
            else
            {
                CartItemProductKit newItem = new CartItemProductKit( CartID, product, optionItemCollection, valueCollection, quantity );
                CartItems.Add( newItem );

                return UpdateResult.Created;
            }
        }

        public DateTime CalcuateRecurringNextPayment(
            DateTime startDate,
            int recurringInterval,
            string recurringIntervalUnit,
            int recurringIntervalNumberOfCycle )
        {
            DateTime nextDate = DateTime.Today;
            for (int i = 1; i <= recurringIntervalNumberOfCycle; i++)
            {
                int nextBilling = 0;
                switch (recurringIntervalUnit.ToLower())
                {
                    case "day":
                        nextBilling = (recurringInterval * i);
                        nextDate = startDate.AddDays( nextBilling );
                        if (nextDate > DateTime.Today)
                        {
                            return nextDate;
                        }
                        break;
                    case "week":
                        nextBilling = ((recurringInterval * 7) * i);
                        nextDate = startDate.AddDays( nextBilling );
                        if (nextDate > DateTime.Today)
                        {
                            return nextDate;
                        }
                        break;
                    case "month":
                        nextBilling = (recurringInterval * i);
                        nextDate = startDate.AddMonths( nextBilling );
                        if (nextDate > DateTime.Today)
                        {
                            return nextDate;
                        }
                        break;
                    case "year":
                        nextBilling = (recurringInterval * i);
                        nextDate = startDate.AddYears( nextBilling );
                        if (nextDate > DateTime.Today)
                        {
                            return nextDate;
                        }
                        break;
                }
            }
            return nextDate;
        }

        public void AddItem( ICartItem cartItem )
        {
            CartItems.Add( cartItem );
        }

        public void AddItem( Product product, int quantity )
        {
            AddItem( product, quantity, OptionItemValueCollection.Null, CartItemGiftDetails.Null, 0, false );
        }

        public void AddItem( Product product, int quantity, bool isSubscriptionable )
        {
            AddItem( product, quantity, OptionItemValueCollection.Null, CartItemGiftDetails.Null, 0, isSubscriptionable );
        }

        public void AddItem(
            Product product,
            int quantity,
            OptionItemValueCollection optionItemCollection )
        {
            AddItem( product, quantity, optionItemCollection, CartItemGiftDetails.Null, 0, false );
        }

        public void AddItem(
            Product product,
            int quantity,
            CartItemGiftDetails giftDetails,
            bool isSubscriptionable )
        {
            AddItem( product, quantity, OptionItemValueCollection.Null, giftDetails, 0, isSubscriptionable );
        }

        public void AddItem(
            Product product,
            int quantity,
            decimal productCustomPrice,
            bool isSubscriptionable )
        {
            AddItem( product, quantity, OptionItemValueCollection.Null, CartItemGiftDetails.Null, productCustomPrice, isSubscriptionable );
        }

        public void AddItem(
            Product product,
            int quantity,
            OptionItemValueCollection optionItemCollection,
            decimal productCustomPrice )
        {
            AddItem( product, quantity, optionItemCollection, CartItemGiftDetails.Null, productCustomPrice, false );
        }

        public void AddItem(
            Product product,
            int quantity,
            OptionItemValueCollection optionItemCollection,
            CartItemGiftDetails giftDetails )
        {
            AddItem( product, quantity, optionItemCollection, giftDetails, 0, false );
        }

        public virtual void AddItem(
            Product product,
            int quantity,
            OptionItemValueCollection optionItemCollection,
            CartItemGiftDetails giftDetails,
            decimal productCustomPrice,
            bool isSubscriptionable )
        {
            int newQuantity =
                GetCurrentQuantity( product.ProductID, optionItemCollection, giftDetails, productCustomPrice )
                + quantity;

            UpdateQuantity( product, newQuantity, optionItemCollection, giftDetails, productCustomPrice, isSubscriptionable );
        }


        public virtual void AddItemByAdmin(
            Product product,
            int quantity,
            OptionItemValueCollection optionItemCollection,
            CartItemGiftDetails giftDetails,
            decimal productCustomPrice,
            decimal EnteredPrice,
            bool isSubscriptionable )
        {
            int newQuantity =
                GetCurrentQuantity( product.ProductID, optionItemCollection, giftDetails, productCustomPrice, EnteredPrice )
                + quantity;

            UpdateQuantity( product, newQuantity, optionItemCollection, giftDetails, productCustomPrice, EnteredPrice, isSubscriptionable );
        }

        public virtual void AddProductKitItem(
            Product product,
            int quantity,
            OptionItemValueCollection optionItemCollection,
            ProductKitItemValueCollection productKitItemCollection,
            decimal productCustomPrice )
        {
            int newQuantity =
                GetCurrentQuantity( product.ProductID, optionItemCollection, productKitItemCollection, productCustomPrice )
                    + quantity;

            UpdateQuantity( product, newQuantity, optionItemCollection, productKitItemCollection, productCustomPrice );
        }

        public virtual void AddProductKitItem(
            Product product,
            int quantity,
            OptionItemValueCollection optionItemCollection,
            ProductKitItemValueCollection productKitItemCollection )
        {
            int newQuantity =
                GetCurrentQuantity( product.ProductID, optionItemCollection, productKitItemCollection )
        + quantity;

            UpdateQuantity( product, newQuantity, optionItemCollection, productKitItemCollection );
        }

        public UpdateResult UpdateQuantity(
            string cartItemID,
            Product product,
            int quantity,
            OptionItemValueCollection optionItemCollection,
            CartItemGiftDetails giftDetails,
            string giftRegistryItemID )
        {
            ICartItem item = FindItemInCart( cartItemID );

            return UpdateQuantity(
                item, product, quantity, optionItemCollection,
                giftDetails, giftRegistryItemID );
        }

        public UpdateResult UpdateQuantity(
            string cartItemID,
            Product product,
            int quantity,
            OptionItemValueCollection optionItemCollection,
            CartItemGiftDetails giftDetails,
            decimal productCustomPrice )
        {
            ICartItem item = FindItemInCart( cartItemID );

            return UpdateQuantity(
                item, product, quantity, optionItemCollection, giftDetails, productCustomPrice, false );
        }

        public UpdateResult UpdateQuantity(
            ICartItem cartItem,
            int quantity )
        {
            if (quantity <= 0)
            {
                DeleteItem( cartItem );
                return UpdateResult.Deleted;
            }
            else
            {
                cartItem.Quantity = quantity;
                return UpdateResult.Updated;
            }
        }

        public UpdateResult UpdateQuantity(
            string cartItemID,
            int quantity )
        {
            ICartItem cartItem = FindItemInCart( cartItemID );
            return UpdateQuantity( cartItem, quantity );
        }

        public UpdateResult DeleteItem( string cartItemID )
        {
            ICartItem item = FindItemInCart( cartItemID );

            return DeleteItem( item );
        }

        public ICartItem FindCartItemByID( string cartItemID )
        {
            return FindItemInCart( cartItemID );
        }

        public CartItemGroup GetAllCartItems()
        {
            return new CartItemGroup( GetCartItems() );
        }

        public ICartItem[] GetCartItems()
        {
            ICartItem[] result = new ICartItem[CartItems.Count];
            for (int i = 0; i < CartItems.Count; i++)
            {
                result[i] = CartItems[i];
            }
            return result;
        }

        public ICartItem[] GetCartItemsWithoutRecurring()
        {
            int recurringCount = 0;
            ArrayList tempCart = new ArrayList();
            foreach (ICartItem items in CartItems)
            {
                if (items.IsRecurring)
                {
                    recurringCount++;
                }
                else
                    tempCart.Add( items );
            }

            ICartItem[] result = new ICartItem[tempCart.Count];
            for (int i = 0; i < tempCart.Count; i++)
            {
                result[i] = (ICartItem) tempCart[i];
            }
            return result;
        }

        public int GetCurrentQuantity()
        {
            int sum = 0;
            foreach (ICartItem item in CartItems)
            {
                sum += item.Quantity;
            }
            return sum;
        }

        public bool ContainsSubscriptionProduct()
        {
            foreach (ICartItem item in CartItems)
            {
                if (item.IsSubscriptionable)
                    return true;
            }
            return false;
        }

        public bool ContainsRecurringProduct()
        {
            foreach (ICartItem item in CartItems)
            {
                if (item.IsRecurring)
                    return true;
            }
            return false;
        }

        public bool ContainsNonRecurringProduct()
        {
            foreach (ICartItem item in CartItems)
            {
                if (!item.IsRecurring)
                    return true;
            }
            return false;
        }

        public bool ContainsFreeShippingCostProduct()
        {
            foreach (ICartItem item in CartItems)
            {
                if (!item.IsPromotion)
                {
                    return item.Product.FreeShippingCost;
                }
            }
            return false;
        }

        public bool CheckCanAddItemToCart( Product item)
        {
            if (CartItems.Count <= 0)
                return true;

            if (ContainsFreeShippingCostProduct() && item.FreeShippingCost)
                return true;

            if (!ContainsFreeShippingCostProduct() && !item.FreeShippingCost)
                return true;

            return false;
        }

        public IList<CartItemGroup> SeparateCartItemGroups()
        {
            IList<CartItemGroup> result = new List<CartItemGroup>();

            IList<ICartItem> nonRecurringItems = new List<ICartItem>();

            foreach (ICartItem cartItem in GetCartItems())
            {
                if (!cartItem.Product.IsRecurring)
                {
                    nonRecurringItems.Add( cartItem );
                }
                else
                {
                    CartItemGroup recurringGroup = new CartItemGroup( cartItem );
                    result.Add( recurringGroup );
                }
            }

            if (nonRecurringItems.Count > 0)
                result.Add( new CartItemGroup( nonRecurringItems ) );

            return result;
        }

        public int NumberOfRecurringProduct()
        {
            int count = 0;

            foreach (ICartItem cartItem in GetCartItems())
            {
                if (cartItem.Product.IsRecurring)
                {
                    count++;
                }
            }

            return count;
        }

        // Subtotal is the sum of product price, not including discount, tax, shipping & handling
        public decimal GetSubtotal( WholesaleStatus wholesaleStatus )
        {
            decimal sum = 0;
            foreach (ICartItem item in CartItems)
            {
                sum += item.GetSubtotal( wholesaleStatus );
            }
            return sum;
        }

        // Subtotal is the sum of product price includes VAT (if the option "Tax Included" is set), not including discount, tax, shipping & handling
        public decimal GetSubtotalIncludedTax( WholesaleStatus wholesaleStatus )
        {
            decimal subtotal = 0;
            foreach (ICartItem item in CartItems)
            {
                subtotal += item.GetSubtotalIncludedTax( wholesaleStatus );
            }
            return subtotal;
        }

        public double GetTotalWeight()
        {
            double sum = 0;
            foreach (ICartItem item in CartItems)
            {
                sum += item.UnitWeight * item.Quantity;
            }
            return sum;
        }

        public int GetNumberOfItems( string productID )
        {
            return GetNumberOfItems( productID, "0" );
        }

        public int GetNumberOfItems( string productID, string optionCombinationID )
        {
            // Separate cart item may have the same OptionCombinationID because not all options
            // may be selected to be used for identifying stock.
            int qty = 0;
            foreach (ICartItem item in GetCartItems())
            {
                if (item.ProductID == productID && item.OptionCombinationID == optionCombinationID)
                    qty += item.Quantity;
            }
            if (optionCombinationID == "0")
            {
                qty += FindSubProductAmountInProductKit( productID );
            }
            //qty += FindSubProductAmountInPromotion( productID, optionCombinationID );
            return qty;
        }

        public bool ContainPromotion()
        {
            foreach (ICartItem item in GetCartItems())
            {
                if (item.IsPromotion)
                    return true;
            }
            return false;
        }

        public bool ContainsShippableItem()
        {
            foreach (ICartItem item in GetCartItems())
            {
                if (item.IsGiftCertificate && item.GiftDetails.NeedPhysical)
                    return true;

                if (!item.IsGiftCertificate && !item.IsDownloadable && !item.IsSubscriptionable)
                    return true;
            }
            return false;
        }



        #endregion

    }

}
