using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Xml.Serialization;
using Vevo.Domain;
using Vevo.QuickBooks.Qbxml70;
using log4net;
using Vevo.Deluxe.Domain.QuickBooks;
using Vevo.Base.Domain;
using Vevo.Deluxe.Domain;


namespace Vevo.QuickBooks.States
{
    /// <summary>
    /// Summary description for QBWCState
    /// </summary>
    public abstract class QBWCState
    {
        private QBWCSession _session;

        private ILog _logger;

        private string GetQbExportedTypeID( QBExportType qbExportType )
        {
            return ((int) qbExportType).ToString();
        }

        protected ILog Logger
        {
            get
            {
                if (_logger == null)
                {
                    _logger = LogManager.GetLogger( typeof( QBWCServices ) );
                }

                return _logger;
            }
        }


        public QBWCSession QBWCSession
        {
            get
            {
                return _session;
            }
        }


        public QBWCState( QBWCSession session )
        {
            _session = session;
        }

        public virtual string OnSendRequestXml( out bool xmlReady )
        {
            string message = String.Format( "Invalid Event: State - {0}, Event - OnSendRequestXml",
                this.GetType().Name );

            throw new ApplicationException( message );
        }

        public virtual int OnReceiveResponseXml( string responseXml )
        {
            string message = String.Format( "Invalid Event: State - {0}, Event - OnReceiveResponseXml",
                this.GetType().Name );

            throw new ApplicationException( message );
        }

        public QBXML CreateRequestQbxml()
        {
            QBXML qbxml = new QBXML();
            qbxml.Items = new object[] { new QBXMLMsgsRq() };
            qbxml.ItemsElementName = new ItemsChoiceType88[] { ItemsChoiceType88.QBXMLMsgsRq };
            return qbxml;
        }

        public string CreateRequestXml( QBXML qbxml )
        {
            StringBuilder sb = new StringBuilder();
            StringWriter sw = new StringWriter( sb );
            XmlSerializer xmlSerializer =
                new XmlSerializer( typeof( QBXML ) );
            xmlSerializer.Serialize( sw, qbxml );
            string xml = sb.ToString();

            xml = xml.Replace(
                @"<?xml version=""1.0"" encoding=""utf-16""?>",
                @"<?xml version=""1.0"" encoding=""Windows-1252""?>" + "\r\n" +
                @"<?qbxml version=""7.0""?>" );

            xml = xml.Replace( @" xmlns:xsi=""http://www.w3.org/2001/XMLSchema-instance""", String.Empty );
            xml = xml.Replace( @" xmlns:xsd=""http://www.w3.org/2001/XMLSchema""", String.Empty );
            return xml;
        }

        public QBXMLMsgsRs CreateResponse( string responseXml )
        {
            StringReader sr = new StringReader( responseXml );

            XmlSerializer xmlSerializer = new XmlSerializer( typeof( QBXML ) );
            QBXML qbxml = (QBXML) xmlSerializer.Deserialize( sr );

            QBXMLMsgsRs qbXMLMsgsRs = (QBXMLMsgsRs) qbxml.Items[0];
            return qbXMLMsgsRs;
        }

        public string CreateUniqueName( string name )
        {
            return CreateUniqueName( name, null );
        }

        public string CreateUniqueName( string name, object listID )
        {
            bool result = false;
            int i = 0;
            string temp = name;
            Dictionary<string, string> fullNameList = QBWCSession.GetNameList();
            while (!result)
            {
                if (fullNameList.ContainsKey( temp ))
                {
                    if (listID != null && fullNameList[temp] == listID.ToString())
                        result = true;
                    else
                    {
                        temp = name + " " + i;
                        i++;
                    }
                }
                else
                    result = true;
            }
            return temp;
        }

        public void CreateResponseConnection()
        {
        }

        public void CreateQBLogSummary( QBExportType qbExportType, int successCount, int failureCount )
        {
            QuickBooksLogSummary log = new QuickBooksLogSummary();
            log.QBLogID = QBWCSession.QBLogID;
            log.QBExportType = qbExportType.ToString();
            log.SuccessCount = successCount;
            log.FailureCount = failureCount;

            QuickBooksLog quickBooksLog = DataAccessContextDeluxe.QuickBooksLogRepository.GetOne( QBWCSession.QBLogID );
            quickBooksLog.QuickBooksLogSummary.Add( log );
            DataAccessContextDeluxe.QuickBooksLogRepository.Save( quickBooksLog );
        }

        public void CreateQBLogItem( string statusCode, string statusSeverity,
            string statusMessage, QBExportType qbExportedType )
        {
            QuickBooksLogItem log = new QuickBooksLogItem();
            log.QBLogID = QBWCSession.QBLogID;
            log.StatusCode = statusCode;
            log.StatusSeverity = statusSeverity;
            log.StatusMessage = statusMessage;
            log.QBExportType = qbExportedType.ToString();

            QuickBooksLog quickBooksLog = DataAccessContextDeluxe.QuickBooksLogRepository.GetOne( QBWCSession.QBLogID );
            quickBooksLog.QuickBooksLogItem.Add( log );
            Logger.Debug( "Logging: CreateQBLogItem LogID:" + QBWCSession.QBLogID );
            DataAccessContextDeluxe.QuickBooksLogRepository.Save( quickBooksLog );
            Logger.Debug( "Logging CreateQBLogItem ID:" + QBWCSession.QBLogID + " Successfully" );
        }
    }

}
