using System;
using System.Data;
using System.Collections.Generic;
using System.Configuration;
using System.Web;


namespace Vevo.WebUI.DataAccessCache
{
    internal interface IDictionaryCache
    {
        void ClearCache();
    }


    /// <summary>
    /// Summary description for DictionaryCache
    /// </summary>
    internal class DictionaryCache<TKey, TValue> : IDictionaryCache
    {
        private string _dictionaryName;


        private Dictionary<TKey, TValue> DataCache
        {
            get
            {
                Dictionary<TKey, TValue> dictionary =
                    (Dictionary<TKey, TValue>) HttpRuntime.Cache[_dictionaryName];

                if (dictionary == null)
                {
                    dictionary = new Dictionary<TKey, TValue>();
                    HttpRuntime.Cache[_dictionaryName] = dictionary;
                }

                return dictionary;
            }
            set
            {
                HttpRuntime.Cache[_dictionaryName] = value;
            }
        }


        public DictionaryCache( string dictionaryName )
        {
            _dictionaryName = dictionaryName;
        }

        public TValue this[TKey key]
        {
            get
            {
                TValue result;
                DataCache.TryGetValue( key, out result );

                return result;
            }
            set
            {
                // Required to duplicate the whole dictionary when modifying collections inside Cache. 
                // The "Cache" entry can be modified as the whole (replacing the cache object).
                // Two concurrent updates may override each other, i.e. only one of these updates is
                // put in the dictionary. However, this is OK for cache. 

                Dictionary<TKey, TValue> newDictionary = new Dictionary<TKey, TValue>( DataCache );
                newDictionary[key] = value;

                DataCache = newDictionary;
            }

        }

        public bool ContainsKey( TKey key )
        {
            return DataCache.ContainsKey( key );
        }

        public void ClearCache()
        {
            HttpRuntime.Cache.Remove( _dictionaryName );
        }

    }
}
