using System;
using System.Data;
using System.Configuration;
using System.IO;
using System.Text;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using Vevo.Domain;
using Vevo.WebAppLib;
using Vevo.WebUI;

namespace Vevo.WebUI.International
{
    /// <summary>
    /// Summary description for BaseLanguageUserControl
    /// </summary>
    public abstract class BaseLanguageUserControl : Vevo.WebUI.BaseControls.BasePublisherUserControl
    {
        private LanguageTokenRule _rule;

        private string ReadHtmlSource()
        {
            StringBuilder sb = new StringBuilder();
            StringWriter sw = new StringWriter( sb );
            HtmlTextWriter htmlWriter = new HtmlTextWriter( sw );
            base.Render( htmlWriter );

            return sb.ToString();
        }

        private PageTokenProcessor CreateTokenProcessor()
        {
            LanguageTokenRule rule = new LanguageTokenRule(
                AppRelativeVirtualPath.Trim( '~' ), 
                StoreContext.Culture.CultureID,
                CultureUtilities.BaseCultureID );
            return new PageTokenProcessor( rule );
        }


        protected string GetLanguageText( string keyword )
        {
            if (_rule == null)
            {
                _rule = new LanguageTokenRule(
                    AppRelativeVirtualPath.Trim( '~' ),
                    CultureUtilities.StoreCultureID,
                    CultureUtilities.BaseCultureID );
            }

            string text = _rule.GetReplacementText( keyword );
            if (String.IsNullOrEmpty( text ))
                return "[$" + keyword + "]";
            else 
            return _rule.GetReplacementText( keyword );
        }

        protected override void Render( HtmlTextWriter writer )
        {
            string source = ReadHtmlSource();

            StringBuilder output = CreateTokenProcessor().Process( source );

            writer.Write( output.ToString() );
        }

        protected StorefrontEvents GetStorefrontEvents()
        {
            if (Page == null)
                return StorefrontEvents.NullInstance;

            if (Page.Master != null &&
                Page.Master is BaseLanguageMasterPage)
                return ((BaseLanguageMasterPage) Page.Master).StorefrontEvents;
            else
                return StorefrontEvents.NullInstance;
        }

    }

}
